import torch
from numpy import resize
from torch.utils.data import Dataset
import numpy as np

class MyDataset(Dataset):
    def __init__(self, data, label, patchsz=1, transform=None):
        '''
        :param data: [h, w, bands]
        :param label: [h, w]
        :param n_components: scale
        :param patchsz: scale
        '''
        super(MyDataset, self).__init__()
        self.data = data  # [h, w, bands]
        self.h, self.w = self.data.shape[0], self.data.shape[1]
        bands = self.data.shape[2]

        self.data_hsi = self.data[:, :, :bands - 1]
        self.data_lidar = self.data[:, :, bands - 1]
        [m, n] = self.data_lidar.shape
        self.data_lidar = self.data_lidar.reshape((m, n, 1))
        self.label = label  # [h, w]
        self.patchsz = patchsz

        self.data_hsi = self.Normalize(self.data_hsi)
        self.data_hsi = self.addMirror(self.data_hsi)
        self.data_lidar = self.Normalize(self.data_lidar)
        self.data_lidar = self.addMirror(self.data_lidar)



    # 数据归一化
    def Normalize(self, data):
        h, w, c = data.shape
        data = data.reshape((h * w, c))
        data -= np.min(data, axis=0)
        data /= np.max(data, axis=0)
        data = data.reshape((h, w, c))
        return data

    # 添加镜像
    def addMirror(self, data):
        dx = self.patchsz // 2
        h, w, bands = data.shape
        mirror = None
        if dx != 0:
            mirror = np.zeros((h + 2 * dx, w + 2 * dx, bands))
            mirror[dx:-dx, dx:-dx, :] = data
            for i in range(dx):
                # 填充左上部分镜像
                mirror[:, i, :] = mirror[:, 2 * dx - i, :]
                mirror[i, :, :] = mirror[2 * dx - i, :, :]
                # 填充右下部分镜像
                mirror[:, -i - 1, :] = mirror[:, -(2 * dx - i) - 1, :]
                mirror[-i - 1, :, :] = mirror[-(2 * dx - i) - 1, :, :]
        return mirror


    def __len__(self):
        return self.h * self. w

    def random_transform(self, a):
        b = a.copy()
        n1 = np.random.randint(0, 2)
        if n1 == 0:
            b1 = self.flip_from_left2right(b).copy()
        elif n1 == 1:
            b1 = self.rotate_matrix_90(b).copy()
        # elif n1 == 2:
        #     b1 = b
        # b2 = self.random_rectangular(b1).copy()
        return b1

    # 随机一个大小为9*9的矩阵mask
    def random_rectangular(self, a):
        patch_mask = 6
        b = a.copy()
        mask = np.zeros_like(b)
        mask[:, :, :] = 1
        h_low = np.random.randint(0, self.patchsz - patch_mask)
        w_low = np.random.randint(0, self.patchsz - patch_mask)
        mask[h_low:h_low+patch_mask, w_low:w_low+patch_mask, :] = 0
        ans = np.multiply(b, mask)
        return ans

    # # 随机点点的mask 掩码率0.2 51 0.5 128 0.75 192 0.35 90
    # def random_point(self, a):
    #     # point_mask = 0.35 * 16 * 16
    #     b = a.copy()
    #     mask = np.zeros_like(b)
    #     mask[:, :, :] = 1
    #     for i in range(192):
    #         h_index = np.random.randint(0, self.patchsz)
    #         w_index = np.random.randint(0, self.patchsz)
    #         mask[h_index, w_index, :] = 0
    #     ans = np.multiply(b, mask)
    #     return ans

    '''矩阵水平翻转'''
    def flip_from_left2right(self, m):
        assert len(m.shape) >= 2
        h, w = m.shape[:2]
        ans = np.zeros_like(m)
        x_coor = np.arange(h).reshape((h, 1))
        y_coor = w - np.arange(w).reshape((1, w)) - 1
        x_coor, y_coor = np.repeat(x_coor, w, 1), np.repeat(y_coor, h, 0)
        ans[x_coor, y_coor] = m
        return ans

    '''矩阵逆时针旋转90度'''
    def rotate_matrix_90(self, m):
        assert len(m.shape) >= 2
        h, w = m.shape[:2]
        ans = np.zeros_like(m)
        new_shape = list(range(len(m.shape)))
        new_shape[0], new_shape[1] = new_shape[1], new_shape[0]
        ans = ans.transpose(new_shape)
        x_coor = np.arange(h).reshape((h, 1))
        y_coor = -np.arange(w).reshape((1, w)) + w - 1
        x_coor, y_coor = np.repeat(x_coor, w, 1), np.repeat(y_coor, h, 0)
        ans[y_coor, x_coor] = m
        return ans

    def __getitem__(self, index):
        '''
        :param index:
        :return: 元素光谱信息， 元素的空间信息， 标签
        '''
        l = index // self.w  # 得到行列坐标
        c = index % self.w

        neighbor_hsi = self.data_hsi[l:l + self.patchsz, c:c + self.patchsz, :]
        neighbor_lidar = self.data_lidar[l:l + self.patchsz, c:c + self.patchsz, :]

        # x1 = self.random_transform(neighbor_hsi).copy()
        # x2 = self.random_transform(neighbor_lidar).copy()
        x1 = neighbor_hsi
        x2 = neighbor_lidar
        '''
        neighbor_hsi = self.data_hsi[l:l + self.patchsz, c:c + self.patchsz, :]
        x1 = self.random_transform(neighbor_hsi).copy()
        x2 = self.random_transform(neighbor_hsi).copy()
'''
        # # 强调中心点的值不会被随机区域擦除擦没
        # x1[self.patchsz // 2, self.patchsz // 2, :] = neighbor_hsi[self.patchsz // 2, self.patchsz // 2, :]
        # x2[self.patchsz // 2, self.patchsz // 2, :] = neighbor_lidar[self.patchsz // 2, self.patchsz // 2, :]

        label = self.label[l, c] - 1
        x1 = torch.tensor(x1, dtype=torch.float32)
        x2 = torch.tensor(x2, dtype=torch.float32)
        return [x1, x2], torch.tensor(label, dtype=torch.long)