import torch
import torch.nn as nn
import torch.nn.functional as F
seed = 84
torch.manual_seed(seed)
torch.manual_seed(seed)
torch.cuda.manual_seed(seed)
torch.cuda.manual_seed_all(seed)

class FT(nn.Module):
    def __init__(self, nc):
        super(FT, self).__init__()

        self.conv1 = nn.Conv2d(128, 128, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn1 = nn.BatchNorm2d(128)

        self.conv2 = nn.Conv2d(128, 128, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn2 = nn.BatchNorm2d(128)
        self.shortcut = nn.Sequential()

        self.conv3 = nn.Conv2d(128, 128, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn3 = nn.BatchNorm2d(128)

        self.conv4 = nn.Conv2d(128, 128, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn4 = nn.BatchNorm2d(128)
        self.shortcut = nn.Sequential()

        self.reduce_dimension1 = nn.Sequential(
            nn.Conv2d(128 * 2, 128, kernel_size=3, stride=1, padding=1, bias=False),
            nn.BatchNorm2d(128),
            nn.ReLU(),
        )

        self.reduce_dimension2 = nn.Sequential(
            nn.Conv2d(128 * 2, 128, kernel_size=3, stride=1, padding=1, bias=False),
            nn.BatchNorm2d(128),
            nn.ReLU(),
        )

        self.reduce_dimension3 = nn.Sequential(
            nn.Conv2d(128 * 2, 128, kernel_size=3, stride=1, padding=1, bias=False),
            nn.BatchNorm2d(128),
            nn.ReLU(),
        )

        self.fc = nn.Linear(128, nc)


    def forward(self, x1, x2, x3, x4):

        out = self.bn1(self.conv1(x1))
        out = F.relu(out)
        out = torch.cat((out, x2), 1)
        out = self.reduce_dimension1(out)

        out = self.bn2(self.conv2(out))
        out = torch.cat((out, x3), 1)
        out = self.reduce_dimension2(out)
        out = out + self.shortcut(x1)
        out = F.relu(out)
        

        xx = out
        out = self.bn3(self.conv3(out))
        out = F.relu(out)
        out = torch.cat((out, x4), 1)
        out = self.reduce_dimension3(out)

        out = self.bn4(self.conv4(out))
        out = out + self.shortcut(xx)
        out = F.relu(out)
    
        out = F.avg_pool2d(out, 9)  #
        out = out.view(out.size(0), -1)
        out = self.fc(out)

        return out
