from torch import nn
from .encoder_hl import Encoder_hl
import torch.nn.functional as F

class projection_MLP(nn.Module):
    def __init__(self, in_dim, out_dim, num_layers=2):
        super().__init__()
        hidden_dim = out_dim
        self.num_layers = num_layers

        self.layer1 = nn.Sequential(
            nn.Linear(in_dim, hidden_dim),
            nn.BatchNorm1d(hidden_dim),
            nn.ReLU(inplace=True)
        )

        self.layer2 = nn.Sequential(
            nn.Linear(hidden_dim, out_dim),
            nn.BatchNorm1d(out_dim, affine=False)  # Page:5, Paragraph:2
        )

    def forward(self, x):
        if self.num_layers == 2:
            x = self.layer1(x)
            x = self.layer2(x)
        return x


class prediction_MLP(nn.Module):
    def __init__(self, in_dim=1024):
        super().__init__()
        out_dim = in_dim
        hidden_dim = int(out_dim / 4)

        self.layer1 = nn.Sequential(
            nn.Linear(in_dim, hidden_dim),
            nn.BatchNorm1d(hidden_dim),
            nn.ReLU(inplace=True)
        )
        self.layer2 = nn.Linear(hidden_dim, out_dim)

    def forward(self, x):
        x = self.layer1(x)
        x = self.layer2(x)

        return x


class PT(nn.Module):
    def __init__(self, args, bands):
        super(PT, self).__init__()

        if args.n == 'hsi_lidar':
            self.backbone_hsi = Encoder_hl(bands)
            self.backbone_lidar = Encoder_hl(1)
            self.backbone_hsi.fc = nn.Identity()
            self.backbone_lidar.fc = nn.Identity()
            self.projector = projection_MLP(64, args.feat_dim,
                                                args.num_proj_layers)
            self.ep_hsi = nn.Sequential(
                self.backbone_hsi,
                self.projector
            )

            self.ep_lidar = nn.Sequential(
                self.backbone_lidar,
                self.projector
            )

            self.n = 'hsi_lidar'

        self.predictor = prediction_MLP(args.feat_dim)

    def forward(self, im_aug1, im_aug2):

        # [_, _, th] = im_aug1
        # [_, _, tl] = im_aug2

        if self.n == 'hsi_lidar':
            z1 = self.ep_hsi(im_aug1)
            z2 = self.ep_lidar(im_aug2)
        p1 = self.predictor(z1)
        p2 = self.predictor(z2)

        return {'z1': z1, 'z2': z2, 'p1': p1, 'p2': p2}







